(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs'), require('rxjs/operators'), require('socket.io-client'), require('@angular/common'), require('@angular/common/http'), require('@angular/core')) :
  typeof define === 'function' && define.amd ? define(['exports', 'rxjs', 'rxjs/operators', 'socket.io-client', '@angular/common', '@angular/common/http', '@angular/core'], factory) :
  (factory((global.ngxLaravelEcho = {}),global.Rx,global.Rx.Observable.prototype,global.io,global.ng.common,global.ng.common.http,global.ng.core));
}(this, (function (exports,rxjs,operators,io,common,http,core) { 'use strict';

  /**
   * @fileoverview added by tsickle
   * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
   */
  /** *
   * The token used to inject the config in Angular's DI system
    @type {?} */
  var ECHO_CONFIG = new core.InjectionToken('echo.config');
  /**
   * @hidden
   */
  var /**
   * @hidden
   */
  TypeFormatter = /** @class */ (function () {
      /**
       * Constructs a new formatter instance
       *
       * @param namespace The namespace of the notifications.
       */
      function TypeFormatter(namespace) {
          /**
           * The namespace of the notifications.
           */
          this.namespace = null;
          this.setNamespace(namespace);
      }
      /**
       * Formats the supplied type
       *
       * @param notificationType The FQN of the notification class
       * @returns The optimized type
       */
      /**
       * Formats the supplied type
       *
       * @param {?} notificationType The FQN of the notification class
       * @return {?} The optimized type
       */
      TypeFormatter.prototype.format = /**
       * Formats the supplied type
       *
       * @param {?} notificationType The FQN of the notification class
       * @return {?} The optimized type
       */
      function (notificationType) {
          if (!this.namespace) {
              return notificationType;
          }
          if (notificationType.indexOf(this.namespace) === 0) {
              return notificationType.substr(this.namespace.length);
          }
          return notificationType;
      };
      /**
       * Sets the namespace
       *
       * @param namespace The namespace of the notifications.
       * @returns The instance for chaining
       */
      /**
       * Sets the namespace
       *
       * @param {?} namespace The namespace of the notifications.
       * @return {?} The instance for chaining
       */
      TypeFormatter.prototype.setNamespace = /**
       * Sets the namespace
       *
       * @param {?} namespace The namespace of the notifications.
       * @return {?} The instance for chaining
       */
      function (namespace) {
          this.namespace = namespace;
          return this;
      };
      return TypeFormatter;
  }());
  /**
   * The service class, use this as something like
   * (or use the [[AngularLaravelEchoModule.forRoot]] method):
   *
   * ```js
   * export const echoConfig: SocketIoEchoConfig = {
   *   userModel: 'App.User',
   *   notificationNamespace: 'App\\Notifications',
   *   options: {
   *     broadcaster: 'socket.io',
   *     host: window.location.hostname + ':6001'
   *   }
   * }
   *
   * \@NgModule({
   *   ...
   *   providers: [
   *     ...
   *     EchoService,
   *     { provide: ECHO_CONFIG, useValue: echoConfig }
   *     ...
   *   ]
   *   ...
   * })
   * ```
   *
   * and import it in your component as
   *
   * ```js
   * \@Component({
   * ...
   * })
   * export class ExampleComponent {
   *   constructor(echoService: EchoService) {
   *   }
   * }
   * ```
   */
  var EchoService = /** @class */ (function () {
      /**
       * Create a new service instance.
       *
       * @param ngZone NgZone instance
       * @param config Service configuration
       */
      function EchoService(ngZone, config) {
          var _this = this;
          this.ngZone = ngZone;
          this.config = config;
          this.channels = [];
          this.notificationListeners = {};
          this.userChannelName = null;
          /** @type {?} */
          var options = Object.assign({}, config.options);
          if (options.broadcaster === 'socket.io') {
              options = Object.assign({
                  client: io
              }, options);
          }
          this._echo = /** @type {?} */ ({ options: options });
          this.options = this.echo.connector.options;
          this.typeFormatter = new TypeFormatter(config.notificationNamespace);
          switch (options.broadcaster) {
              case 'null':
                  this.connectionState$ = rxjs.of({ type: 'connected' });
                  break;
              case 'socket.io':
                  this.connectionState$ = new rxjs.Observable(function (subscriber) {
                      /** @type {?} */
                      var socket = (/** @type {?} */ (_this._echo.connector)).socket;
                      /** @type {?} */
                      var handleConnect = function () { return _this.ngZone.run(function () { return subscriber.next({ type: 'connect' }); }); };
                      /** @type {?} */
                      var handleConnectError = function (error) { return _this.ngZone.run(function () { return subscriber.next({ type: 'connect_error', error: error }); }); };
                      /** @type {?} */
                      var handleConnectTimeout = function (timeout) { return _this.ngZone.run(function () { return subscriber.next({ type: 'connect_timeout', timeout: timeout }); }); };
                      /** @type {?} */
                      var handleError = function (error) { return _this.ngZone.run(function () { return subscriber.next({ type: 'error', error: error }); }); };
                      /** @type {?} */
                      var handleDisconnect = function (reason) { return _this.ngZone.run(function () { return subscriber.next({ type: 'disconnect', reason: reason }); }); };
                      /** @type {?} */
                      var handleReconnect = function (attemptNumber) { return _this.ngZone.run(function () { return subscriber.next({ type: 'reconnect', attemptNumber: attemptNumber }); }); };
                      /** @type {?} */
                      var handleReconnectAttempt = function (attemptNumber) { return _this.ngZone.run(function () { return subscriber.next({ type: 'reconnect_attempt', attemptNumber: attemptNumber }); }); };
                      /** @type {?} */
                      var handleReconnecting = function (attemptNumber) { return _this.ngZone.run(function () { return subscriber.next({ type: 'reconnecting', attemptNumber: attemptNumber }); }); };
                      /** @type {?} */
                      var handleReconnectError = function (error) { return _this.ngZone.run(function () { return subscriber.next({ type: 'reconnect_error', error: error }); }); };
                      /** @type {?} */
                      var handleReconnectFailed = function () { return _this.ngZone.run(function () { return subscriber.next({ type: 'reconnect_failed' }); }); };
                      /** @type {?} */
                      var handlePing = function () { return _this.ngZone.run(function () { return subscriber.next({ type: 'ping' }); }); };
                      /** @type {?} */
                      var handlePong = function (latency) { return _this.ngZone.run(function () { return subscriber.next({ type: 'pong', latency: latency }); }); };
                      socket.on('connect', handleConnect);
                      socket.on('connect_error', handleConnectError);
                      socket.on('connect_timeout', handleConnectTimeout);
                      socket.on('error', handleError);
                      socket.on('disconnect', handleDisconnect);
                      socket.on('reconnect', handleReconnect);
                      socket.on('reconnect_attempt', handleReconnectAttempt);
                      socket.on('reconnecting', handleReconnecting);
                      socket.on('reconnect_error', handleReconnectError);
                      socket.on('reconnect_failed', handleReconnectFailed);
                      socket.on('ping', handlePing);
                      socket.on('pong', handlePong);
                      return function () {
                          socket.off('connect', handleConnect);
                          socket.off('connect_error', handleConnectError);
                          socket.off('connect_timeout', handleConnectTimeout);
                          socket.off('error', handleError);
                          socket.off('disconnect', handleDisconnect);
                          socket.off('reconnect', handleReconnect);
                          socket.off('reconnect_attempt', handleReconnectAttempt);
                          socket.off('reconnecting', handleReconnecting);
                          socket.off('reconnect_error', handleReconnectError);
                          socket.off('reconnect_failed', handleReconnectFailed);
                          socket.off('ping', handlePing);
                          socket.off('pong', handlePong);
                      };
                  }).pipe(operators.shareReplay(1));
                  break;
              case 'pusher':
                  this.connectionState$ = new rxjs.Observable(function (subscriber) {
                      /** @type {?} */
                      var socket = (/** @type {?} */ (_this._echo.connector)).pusher.connection;
                      /** @type {?} */
                      var handleStateChange = function (_a) {
                          var current = _a.current;
                          return _this.ngZone.run(function () { return subscriber.next({ type: current }); });
                      };
                      /** @type {?} */
                      var handleConnectingIn = function (delay) { return _this.ngZone.run(function () { return subscriber.next({ type: 'connecting_in', delay: delay }); }); };
                      socket.bind('state_change', handleStateChange);
                      socket.bind('connecting_in', handleConnectingIn);
                      return function () {
                          socket.unbind('state_change', handleStateChange);
                          socket.unbind('connecting_in', handleConnectingIn);
                      };
                  }).pipe(operators.shareReplay(1));
                  break;
              default:
                  this.connectionState$ = rxjs.throwError(new Error('unsupported'));
                  break;
          }
          this.connected$ = (/** @type {?} */ (this.connectionState$)).pipe(operators.map(function () { return _this.connected; }), operators.startWith(this.connected), operators.distinctUntilChanged(), operators.shareReplay(1));
      }
      Object.defineProperty(EchoService.prototype, "connected", {
          /**
           * Is the socket currently connected
           */
          get: /**
           * Is the socket currently connected
           * @return {?}
           */
          function () {
              if (this.options.broadcaster === 'null') {
                  // Null broadcaster is always connected
                  return true;
              }
              if (this.options.broadcaster === 'pusher') {
                  return (/** @type {?} */ (this._echo.connector)).pusher.connection.state === 'connected';
              }
              return (/** @type {?} */ (this._echo.connector)).socket.connected;
          },
          enumerable: true,
          configurable: true
      });
      Object.defineProperty(EchoService.prototype, "connectionState", {
          /**
           * Observable of connection state changes, emits true when connected and false when disconnected
           */
          get: /**
           * Observable of connection state changes, emits true when connected and false when disconnected
           * @return {?}
           */
          function () {
              return this.connected$;
          },
          enumerable: true,
          configurable: true
      });
      Object.defineProperty(EchoService.prototype, "rawConnectionState", {
          /**
           * Observable of raw events of the underlying connection
           */
          get: /**
           * Observable of raw events of the underlying connection
           * @return {?}
           */
          function () {
              return this.connectionState$;
          },
          enumerable: true,
          configurable: true
      });
      Object.defineProperty(EchoService.prototype, "echo", {
          /**
           * The echo instance, can be used to implement any custom requirements outside of this service (remember to include NgZone.run calls)
           */
          get: /**
           * The echo instance, can be used to implement any custom requirements outside of this service (remember to include NgZone.run calls)
           * @return {?}
           */
          function () {
              return this._echo;
          },
          enumerable: true,
          configurable: true
      });
      Object.defineProperty(EchoService.prototype, "socketId", {
          /**
           * The socket ID
           */
          get: /**
           * The socket ID
           * @return {?}
           */
          function () {
              return this.echo.socketId();
          },
          enumerable: true,
          configurable: true
      });
      /**
       * Gets the named and optionally typed channel from the channels array if it exists
       *
       * @param {?} name The name of the channel
       * @param {?=} type The type of channel to lookup
       * @return {?} The channel if found or null
       */
      EchoService.prototype.getChannelFromArray = /**
       * Gets the named and optionally typed channel from the channels array if it exists
       *
       * @param {?} name The name of the channel
       * @param {?=} type The type of channel to lookup
       * @return {?} The channel if found or null
       */
      function (name, type) {
          if (type === void 0) { type = null; }
          /** @type {?} */
          var channel = this.channels.find(function (c) { return c.name === name; });
          if (channel) {
              if (type && channel.type !== type) {
                  throw new Error("Channel " + name + " is not a " + type + " channel");
              }
              return channel;
          }
          return null;
      };
      /**
       * Gets the named and optionally typed channel from the channels array or throws if it does not exist
       *
       * @param {?} name The name of the channel
       * @param {?=} type The type of channel to lookup
       * @return {?} The channel
       */
      EchoService.prototype.requireChannelFromArray = /**
       * Gets the named and optionally typed channel from the channels array or throws if it does not exist
       *
       * @param {?} name The name of the channel
       * @param {?=} type The type of channel to lookup
       * @return {?} The channel
       */
      function (name, type) {
          if (type === void 0) { type = null; }
          /** @type {?} */
          var channel = this.getChannelFromArray(name, type);
          if (!channel) {
              if (type) {
                  throw new Error("" + type[0].toUpperCase() + type.substr(1) + " channel " + name + " does not exist");
              }
              throw new Error("Channel " + name + " does not exist");
          }
          return channel;
      };
      /**
       * Fetch or create a public channel
       *
       * @param {?} name The name of the channel to join
       * @return {?} The fetched or created channel
       */
      EchoService.prototype.publicChannel = /**
       * Fetch or create a public channel
       *
       * @param {?} name The name of the channel to join
       * @return {?} The fetched or created channel
       */
      function (name) {
          /** @type {?} */
          var channel = this.getChannelFromArray(name, 'public');
          if (channel) {
              return channel.channel;
          }
          /** @type {?} */
          var echoChannel = this.echo.channel(name);
          channel = {
              name: name,
              channel: echoChannel,
              type: 'public',
              listeners: {},
          };
          this.channels.push(channel);
          return echoChannel;
      };
      /**
       * Fetch or create a presence channel and subscribe to the presence events
       *
       * @param {?} name The name of the channel to join
       * @return {?} The fetched or created channel
       */
      EchoService.prototype.presenceChannel = /**
       * Fetch or create a presence channel and subscribe to the presence events
       *
       * @param {?} name The name of the channel to join
       * @return {?} The fetched or created channel
       */
      function (name) {
          var _this = this;
          /** @type {?} */
          var channel = this.getChannelFromArray(name, 'presence');
          if (channel) {
              return /** @type {?} */ (channel.channel);
          }
          /** @type {?} */
          var echoChannel = this.echo.join(name);
          channel = {
              name: name,
              channel: echoChannel,
              type: 'presence',
              listeners: {},
              users: null,
          };
          this.channels.push(channel);
          echoChannel.here(function (users) {
              _this.ngZone.run(function () {
                  if (channel) {
                      channel.users = users;
                      if (channel.listeners['_users_']) {
                          channel.listeners['_users_'].next(JSON.parse(JSON.stringify(users)));
                      }
                  }
              });
          });
          echoChannel.joining(function (user) {
              _this.ngZone.run(function () {
                  if (channel) {
                      channel.users = channel.users || [];
                      channel.users.push(user);
                      if (channel.listeners['_joining_']) {
                          channel.listeners['_joining_'].next(JSON.parse(JSON.stringify(user)));
                      }
                  }
              });
          });
          echoChannel.leaving(function (user) {
              _this.ngZone.run(function () {
                  if (channel) {
                      channel.users = channel.users || [];
                      /** @type {?} */
                      var existing = channel.users.find(function (e) { return e === user; });
                      if (existing) {
                          /** @type {?} */
                          var index = channel.users.indexOf(existing);
                          if (index !== -1) {
                              channel.users.splice(index, 1);
                          }
                      }
                      if (channel.listeners['_leaving_']) {
                          channel.listeners['_leaving_'].next(JSON.parse(JSON.stringify(user)));
                      }
                  }
              });
          });
          return echoChannel;
      };
      /**
       * Fetch or create a private channel
       *
       * @param {?} name The name of the channel to join
       * @return {?} The fetched or created channel
       */
      EchoService.prototype.privateChannel = /**
       * Fetch or create a private channel
       *
       * @param {?} name The name of the channel to join
       * @return {?} The fetched or created channel
       */
      function (name) {
          /** @type {?} */
          var channel = this.getChannelFromArray(name, 'private');
          if (channel) {
              return /** @type {?} */ (channel.channel);
          }
          /** @type {?} */
          var echoChannel = this.echo.private(name);
          channel = {
              name: name,
              channel: echoChannel,
              type: 'private',
              listeners: {},
          };
          this.channels.push(channel);
          return echoChannel;
      };
      /**
       * Set authentication data and connect to and start listening for notifications on the users private channel
       *
       * @param headers Authentication headers to send when talking to the service
       * @param userId The current user's id
       * @returns The instance for chaining
       */
      /**
       * Set authentication data and connect to and start listening for notifications on the users private channel
       *
       * @param {?} headers Authentication headers to send when talking to the service
       * @param {?} userId The current user's id
       * @return {?} The instance for chaining
       */
      EchoService.prototype.login = /**
       * Set authentication data and connect to and start listening for notifications on the users private channel
       *
       * @param {?} headers Authentication headers to send when talking to the service
       * @param {?} userId The current user's id
       * @return {?} The instance for chaining
       */
      function (headers, userId) {
          var _this = this;
          /** @type {?} */
          var newChannelName = this.config.userModel.replace('\\', '.') + "." + userId;
          if (this.userChannelName && this.userChannelName !== newChannelName) {
              this.logout();
          }
          this.options.auth = this.options.auth || {};
          this.options.auth.headers = Object.assign({}, headers);
          if (this.options.broadcaster === 'pusher') {
              /** @type {?} */
              var connector = (/** @type {?} */ (this._echo.connector));
              if (connector.pusher.config.auth !== this.options.auth) {
                  connector.pusher.config.auth = this.options.auth;
              }
          }
          if (this.userChannelName !== newChannelName) {
              this.userChannelName = newChannelName;
              this.privateChannel(newChannelName).notification(function (notification) {
                  /** @type {?} */
                  var type = _this.typeFormatter.format(notification.type);
                  if (_this.notificationListeners[type]) {
                      _this.ngZone.run(function () { return _this.notificationListeners[type].next(notification); });
                  }
                  if (_this.notificationListeners['*']) {
                      _this.ngZone.run(function () { return _this.notificationListeners['*'].next(notification); });
                  }
              });
          }
          return this;
      };
      /**
       * Clear authentication data and close any presence or private channels.
       *
       * @returns The instance for chaining
       */
      /**
       * Clear authentication data and close any presence or private channels.
       *
       * @return {?} The instance for chaining
       */
      EchoService.prototype.logout = /**
       * Clear authentication data and close any presence or private channels.
       *
       * @return {?} The instance for chaining
       */
      function () {
          var _this = this;
          this.channels
              .filter(function (channel) { return channel.type !== 'public'; })
              .forEach(function (channel) { return _this.leave(channel.name); });
          this.options.auth = this.options.auth || {};
          this.options.auth.headers = {};
          return this;
      };
      /**
       * Join a channel of specified name and type.
       *
       * @param name The name of the channel to join
       * @param type The type of channel to join
       * @returns The instance for chaining
       */
      /**
       * Join a channel of specified name and type.
       *
       * @param {?} name The name of the channel to join
       * @param {?} type The type of channel to join
       * @return {?} The instance for chaining
       */
      EchoService.prototype.join = /**
       * Join a channel of specified name and type.
       *
       * @param {?} name The name of the channel to join
       * @param {?} type The type of channel to join
       * @return {?} The instance for chaining
       */
      function (name, type) {
          switch (type) {
              case 'public':
                  this.publicChannel(name);
                  break;
              case 'presence':
                  this.presenceChannel(name);
                  break;
              case 'private':
                  this.privateChannel(name);
                  break;
          }
          return this;
      };
      /**
       * Leave a channel of the specified name.
       *
       * @param name The name of the channel to leave
       * @returns The instance for chaining
       */
      /**
       * Leave a channel of the specified name.
       *
       * @param {?} name The name of the channel to leave
       * @return {?} The instance for chaining
       */
      EchoService.prototype.leave = /**
       * Leave a channel of the specified name.
       *
       * @param {?} name The name of the channel to leave
       * @return {?} The instance for chaining
       */
      function (name) {
          /** @type {?} */
          var channel = this.getChannelFromArray(name);
          if (channel) {
              this.echo.leave(name);
              Object.keys(channel.listeners).forEach(function (key) { return channel.listeners[key].complete(); });
              if (channel.notificationListeners) {
                  Object.keys(channel.notificationListeners).forEach(function (key) { return channel.notificationListeners && channel.notificationListeners[key].complete(); });
              }
              /** @type {?} */
              var index = this.channels.indexOf(channel);
              if (index !== -1) {
                  this.channels.splice(index, 1);
              }
          }
          return this;
      };
      /**
       * Listen for events on the specified channel.
       *
       * @param name The name of the channel
       * @param event The name of the event
       * @returns An observable that emits the event data of the specified event when it arrives
       */
      /**
       * Listen for events on the specified channel.
       *
       * @param {?} name The name of the channel
       * @param {?} event The name of the event
       * @return {?} An observable that emits the event data of the specified event when it arrives
       */
      EchoService.prototype.listen = /**
       * Listen for events on the specified channel.
       *
       * @param {?} name The name of the channel
       * @param {?} event The name of the event
       * @return {?} An observable that emits the event data of the specified event when it arrives
       */
      function (name, event) {
          var _this = this;
          /** @type {?} */
          var channel = this.requireChannelFromArray(name);
          if (!channel.listeners[event]) {
              /** @type {?} */
              var listener_1 = new rxjs.Subject();
              channel.channel.listen(event, function (e) { return _this.ngZone.run(function () { return listener_1.next(e); }); });
              channel.listeners[event] = listener_1;
          }
          return channel.listeners[event].asObservable();
      };
      /**
       * Listen for client sent events (whispers) on the specified private or presence channel channel.
       *
       * @param name The name of the channel
       * @param event The name of the event
       * @returns An observable that emits the whisper data of the specified event when it arrives
       */
      /**
       * Listen for client sent events (whispers) on the specified private or presence channel channel.
       *
       * @param {?} name The name of the channel
       * @param {?} event The name of the event
       * @return {?} An observable that emits the whisper data of the specified event when it arrives
       */
      EchoService.prototype.listenForWhisper = /**
       * Listen for client sent events (whispers) on the specified private or presence channel channel.
       *
       * @param {?} name The name of the channel
       * @param {?} event The name of the event
       * @return {?} An observable that emits the whisper data of the specified event when it arrives
       */
      function (name, event) {
          var _this = this;
          /** @type {?} */
          var channel = this.requireChannelFromArray(name);
          if (channel.type === 'public') {
              return rxjs.throwError(new Error('Whisper is not available on public channels'));
          }
          if (!channel.listeners["_whisper_" + event + "_"]) {
              /** @type {?} */
              var listener_2 = new rxjs.Subject();
              channel.channel.listenForWhisper(event, function (e) { return _this.ngZone.run(function () { return listener_2.next(e); }); });
              channel.listeners["_whisper_" + event + "_"] = listener_2;
          }
          return channel.listeners["_whisper_" + event + "_"].asObservable();
      };
      /**
       * Listen for notifications on the users private channel.
       *
       * @param type The type of notification to listen for or `*` for any
       * @param name Optional a different channel to receive notifications on
       * @returns An observable that emits the notification of the specified type when it arrives
       */
      /**
       * Listen for notifications on the users private channel.
       *
       * @param {?} type The type of notification to listen for or `*` for any
       * @param {?=} name Optional a different channel to receive notifications on
       * @return {?} An observable that emits the notification of the specified type when it arrives
       */
      EchoService.prototype.notification = /**
       * Listen for notifications on the users private channel.
       *
       * @param {?} type The type of notification to listen for or `*` for any
       * @param {?=} name Optional a different channel to receive notifications on
       * @return {?} An observable that emits the notification of the specified type when it arrives
       */
      function (type, name) {
          var _this = this;
          type = this.typeFormatter.format(type);
          if (name && name !== this.userChannelName) {
              /** @type {?} */
              var channel_1 = this.requireChannelFromArray(name);
              if (!channel_1.notificationListeners) {
                  channel_1.notificationListeners = {};
                  channel_1.channel.notification(function (notification) {
                      /** @type {?} */
                      var notificationType = _this.typeFormatter.format(notification.type);
                      if (channel_1.notificationListeners) {
                          if (channel_1.notificationListeners[notificationType]) {
                              _this.ngZone.run(function () { return channel_1.notificationListeners && channel_1.notificationListeners[notificationType].next(notification); });
                          }
                          if (channel_1.notificationListeners['*']) {
                              _this.ngZone.run(function () { return channel_1.notificationListeners && channel_1.notificationListeners['*'].next(notification); });
                          }
                      }
                  });
              }
              if (!channel_1.notificationListeners[type]) {
                  channel_1.notificationListeners[type] = new rxjs.Subject();
              }
              return channel_1.notificationListeners[type].asObservable();
          }
          if (!this.notificationListeners[type]) {
              this.notificationListeners[type] = new rxjs.Subject();
          }
          return this.notificationListeners[type].asObservable();
      };
      /**
       * Listen for users joining the specified presence channel.
       *
       * @param name The name of the channel
       * @returns An observable that emits the user when he joins the specified channel
       */
      /**
       * Listen for users joining the specified presence channel.
       *
       * @param {?} name The name of the channel
       * @return {?} An observable that emits the user when he joins the specified channel
       */
      EchoService.prototype.joining = /**
       * Listen for users joining the specified presence channel.
       *
       * @param {?} name The name of the channel
       * @return {?} An observable that emits the user when he joins the specified channel
       */
      function (name) {
          /** @type {?} */
          var channel = this.requireChannelFromArray(name, 'presence');
          if (!channel.listeners["_joining_"]) {
              channel.listeners['_joining_'] = new rxjs.Subject();
          }
          return channel.listeners['_joining_'].asObservable();
      };
      /**
       * Listen for users leaving the specified presence channel.
       *
       * @param name The name of the channel
       * @returns An observable that emits the user when he leaves the specified channel
       */
      /**
       * Listen for users leaving the specified presence channel.
       *
       * @param {?} name The name of the channel
       * @return {?} An observable that emits the user when he leaves the specified channel
       */
      EchoService.prototype.leaving = /**
       * Listen for users leaving the specified presence channel.
       *
       * @param {?} name The name of the channel
       * @return {?} An observable that emits the user when he leaves the specified channel
       */
      function (name) {
          /** @type {?} */
          var channel = this.requireChannelFromArray(name, 'presence');
          if (!channel.listeners["_leaving_"]) {
              channel.listeners['_leaving_'] = new rxjs.Subject();
          }
          return channel.listeners['_leaving_'].asObservable();
      };
      /**
       * Listen for user list updates on the specified presence channel.
       *
       * @param name The name of the channel
       * @returns An observable that emits the initial user list as soon as it's available
       */
      /**
       * Listen for user list updates on the specified presence channel.
       *
       * @param {?} name The name of the channel
       * @return {?} An observable that emits the initial user list as soon as it's available
       */
      EchoService.prototype.users = /**
       * Listen for user list updates on the specified presence channel.
       *
       * @param {?} name The name of the channel
       * @return {?} An observable that emits the initial user list as soon as it's available
       */
      function (name) {
          /** @type {?} */
          var channel = this.requireChannelFromArray(name, 'presence');
          if (!channel.listeners["_users_"]) {
              channel.listeners['_users_'] = new rxjs.ReplaySubject(1);
          }
          return channel.listeners['_users_'].asObservable();
      };
      /**
       * Send a client event to the specified presence or private channel (whisper).
       *
       * @param name The name of the channel
       * @param event The name of the event
       * @param data The payload for the event
       * @returns The instance for chaining
       */
      /**
       * Send a client event to the specified presence or private channel (whisper).
       *
       * @param {?} name The name of the channel
       * @param {?} event The name of the event
       * @param {?} data The payload for the event
       * @return {?} The instance for chaining
       */
      EchoService.prototype.whisper = /**
       * Send a client event to the specified presence or private channel (whisper).
       *
       * @param {?} name The name of the channel
       * @param {?} event The name of the event
       * @param {?} data The payload for the event
       * @return {?} The instance for chaining
       */
      function (name, event, data) {
          /** @type {?} */
          var channel = this.requireChannelFromArray(name);
          if (channel.type === 'public') {
              throw new Error('Whisper is not available on public channels');
          }
          /** @type {?} */
          var echoChannel = /** @type {?} */ (channel.channel);
          echoChannel.whisper(event, data);
          return this;
      };
      EchoService.decorators = [
          { type: core.Injectable },
      ];
      /** @nocollapse */
      EchoService.ctorParameters = function () { return [
          { type: core.NgZone },
          { type: undefined, decorators: [{ type: core.Inject, args: [ECHO_CONFIG,] }] }
      ]; };
      return EchoService;
  }());

  /**
   * @fileoverview added by tsickle
   * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
   */

  /**
   * @fileoverview added by tsickle
   * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
   */
  /**
   * An http interceptor to automatically add the socket ID header, use this as something like
   * (or use the [[AngularLaravelEchoModule.forRoot]] method):
   *
   * ```js
   * \@NgModule({
   *   ...
   *   providers: [
   *     ...
   *     { provide: HTTP_INTERCEPTORS, useClass: EchoInterceptor, multi: true }
   *     ...
   *   ]
   *   ...
   * })
   * ```
   */
  var EchoInterceptor = /** @class */ (function () {
      function EchoInterceptor(echoService) {
          this.echoService = echoService;
      }
      /**
       * @param {?} req
       * @param {?} next
       * @return {?}
       */
      EchoInterceptor.prototype.intercept = /**
       * @param {?} req
       * @param {?} next
       * @return {?}
       */
      function (req, next) {
          /** @type {?} */
          var socketId = this.echoService.socketId;
          if (this.echoService.connected && socketId) {
              req = req.clone({ headers: req.headers.append('X-Socket-ID', socketId) });
          }
          return next.handle(req);
      };
      EchoInterceptor.decorators = [
          { type: core.Injectable },
      ];
      /** @nocollapse */
      EchoInterceptor.ctorParameters = function () { return [
          { type: EchoService }
      ]; };
      return EchoInterceptor;
  }());

  /**
   * @fileoverview added by tsickle
   * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
   */
  /**
   * Module definition, use [[forRoot]] for easy configuration
   * of the service and interceptor
   */
  var AngularLaravelEchoModule = /** @class */ (function () {
      function AngularLaravelEchoModule() {
      }
      /**
       * Make the service and interceptor available for the current (root) module, it is recommended that this method
       * is only called from the root module otherwise multiple instances of the service and interceptor will be created
       * (one for each module it is called in)
       * @param {?} config
       * @return {?}
       */
      AngularLaravelEchoModule.forRoot = /**
       * Make the service and interceptor available for the current (root) module, it is recommended that this method
       * is only called from the root module otherwise multiple instances of the service and interceptor will be created
       * (one for each module it is called in)
       * @param {?} config
       * @return {?}
       */
      function (config) {
          return {
              ngModule: AngularLaravelEchoModule,
              providers: [
                  EchoService,
                  { provide: http.HTTP_INTERCEPTORS, useClass: EchoInterceptor, multi: true },
                  { provide: ECHO_CONFIG, useValue: config },
              ]
          };
      };
      AngularLaravelEchoModule.decorators = [
          { type: core.NgModule, args: [{
                      imports: [common.CommonModule],
                  },] },
      ];
      return AngularLaravelEchoModule;
  }());

  /**
   * @fileoverview added by tsickle
   * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
   */

  /**
   * @fileoverview added by tsickle
   * @suppress {checkTypes,extraRequire,uselessCode} checked by tsc
   */

  exports.EchoService = EchoService;
  exports.EchoInterceptor = EchoInterceptor;
  exports.AngularLaravelEchoModule = AngularLaravelEchoModule;
  exports.ECHO_CONFIG = ECHO_CONFIG;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
